"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createMongoDBContext = void 0;
const mongodb_1 = require("mongodb");
exports.createMongoDBContext = (Config, Options) => {
    const ConnectClient = (() => {
        let _DBClient;
        return async () => {
            if (_DBClient === undefined) {
                const URI = Config.connection;
                const client = new mongodb_1.MongoClient(URI, Options);
                await client.connect();
                _DBClient = client;
            }
            return _DBClient;
        };
    })();
    const ConnectDB = async () => {
        const client = await ConnectClient();
        const { dbName } = Config;
        const DBInstance = client.db(dbName);
        return DBInstance;
    };
    const ConnectCollection = async (collection) => {
        const DBInstance = await ConnectDB();
        return DBInstance.collection(collection);
    };
    const dataContext = {
        client: async () => ConnectClient(),
        db: async () => ConnectDB(),
        newID: () => new mongodb_1.ObjectID(),
        close: async () => {
            const client = await ConnectClient();
            client.close(true);
        },
        createItem: async (collection, item, options = {}) => {
            const Collection = await ConnectCollection(collection);
            const { result } = await Collection.insertOne(item, options);
            return { affected: result.n };
        },
        queryByID: async (collection, id, options = {}) => {
            const Collection = await ConnectCollection(collection);
            const filter = { _id: id };
            const item = await Collection.findOne(filter, options);
            if (item === undefined || item === null) {
                return undefined;
            }
            else {
                return item;
            }
        },
        removeByID: async (collection, id, options = {}) => {
            const Collection = await ConnectCollection(collection);
            const filter = { _id: id };
            const { result } = await Collection.deleteOne(filter, options);
            return { affected: result.n };
        },
        toRepository: (collection, defaultOptions = {}) => ({
            collection: async () => ConnectCollection(collection),
            createItem: (item, options = {}) => dataContext.createItem(collection, item, Object.assign({}, defaultOptions, options)),
            newID: () => dataContext.newID(),
            queryByID: (id, options = {}) => dataContext.queryByID(collection, id, Object.assign({}, defaultOptions, options)),
            removeByID: (id, options = {}) => dataContext.removeByID(collection, id, Object.assign({}, defaultOptions, options)),
        }),
    };
    return dataContext;
};
Object.freeze(exports.createMongoDBContext);
//# sourceMappingURL=createMongoDBContext.js.map